//---------------------------------------------------------------------------
// Copyright 1997-2020 Joe Lowe
//
// Permission is granted to any person obtaining a copy of this Software,
// to deal in the Software without restriction, including the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and sell copies of
// the Software.
//
// The above copyright and permission notice must be left intact in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS WITHOUT WARRANTY.
//---------------------------------------------------------------------------
// file name:  portableint.h
// created:    1997.06.20
//---------------------------------------------------------------------------
#ifndef PORTABLEINT_H
#define PORTABLEINT_H
#include "ptlibc.h"
#include "bitops.h"
PT_EXTERNC_START

   // No portable way to determine byte order in preprocessor.
   // Must be pre-defined, either on compiler command line or
   // from previous definition in another header.
#if !defined(PT_LITTLEENDIAN) && !defined(PT_BIGENDIAN)
   #error Unable to determine byte order.
#endif

PT_INLINE  int16_t  pt_swapint16(  int16_t s) { return PT_SCAST(int16_t,bits_swap16(PT_SCAST(uint16_t,s))); }
PT_INLINE  int32_t  pt_swapint32(  int32_t s) { return PT_SCAST(int32_t,bits_swap32(PT_SCAST(uint32_t,s))); }
PT_INLINE  int64_t  pt_swapint64(  int64_t s) { return PT_SCAST(int64_t,bits_swap64(PT_SCAST(uint64_t,s))); }
PT_INLINE uint16_t pt_swapuint16( uint16_t s) { return bits_swap16(s); }
PT_INLINE uint32_t pt_swapuint32( uint32_t s) { return bits_swap32(s); }
PT_INLINE uint64_t pt_swapuint64( uint64_t s) { return bits_swap64(s); }

#ifdef PT_LITTLEENDIAN
   typedef     int16_t     pt_int16le_t ;
   typedef     int32_t     pt_int32le_t ;
   typedef     int64_t     pt_int64le_t ;
   typedef    uint16_t    pt_uint16le_t ;
   typedef    uint32_t    pt_uint32le_t ;
   typedef    uint64_t    pt_uint64le_t ;
   typedef  pt_int16u_t    pt_int16leu_t;
   typedef  pt_int32u_t    pt_int32leu_t;
   typedef  pt_int64u_t    pt_int64leu_t;
   typedef pt_uint16u_t   pt_uint16leu_t;
   typedef pt_uint32u_t   pt_uint32leu_t;
   typedef pt_uint64u_t   pt_uint64leu_t;
   typedef  pt_int16oe_t   pt_int16be_t ;
   typedef  pt_int32oe_t   pt_int32be_t ;
   typedef  pt_int64oe_t   pt_int64be_t ;
   typedef pt_uint16oe_t  pt_uint16be_t ;
   typedef pt_uint32oe_t  pt_uint32be_t ;
   typedef pt_uint64oe_t  pt_uint64be_t ;
   typedef  pt_int16oeu_t  pt_int16beu_t;
   typedef  pt_int32oeu_t  pt_int32beu_t;
   typedef  pt_int64oeu_t  pt_int64beu_t;
   typedef pt_uint16oeu_t pt_uint16beu_t;
   typedef pt_uint32oeu_t pt_uint32beu_t;
   typedef pt_uint64oeu_t pt_uint64beu_t;
   typedef pt_uuid_t    pt_uuidle_t ;
   typedef pt_uuidu_t   pt_uuidleu_t;
   typedef pt_uuidoe_t  pt_uuidbe_t ;
   typedef pt_uuidoeu_t pt_uuidbeu_t;
   #define  INT16LE      int16_t
   #define  INT32LE      int32_t
   #define  INT64LE      int64_t
   #define UINT16LE     uint16_t
   #define UINT32LE     uint32_t
   #define UINT64LE     uint64_t
   #define  INT16LEU  pt_int16u_t
   #define  INT32LEU  pt_int32u_t
   #define  INT64LEU  pt_int64u_t
   #define UINT16LEU pt_uint16u_t
   #define UINT32LEU pt_uint32u_t
   #define UINT64LEU pt_uint64u_t
   #define  INT16BE   pt_int16oe_t
   #define  INT32BE   pt_int32oe_t
   #define  INT64BE   pt_int64oe_t
   #define UINT16BE  pt_uint16oe_t
   #define UINT32BE  pt_uint32oe_t
   #define UINT64BE  pt_uint64oe_t
   #define  INT16BEU  pt_int16oeu_t
   #define  INT32BEU  pt_int32oeu_t
   #define  INT64BEU  pt_int64oeu_t
   #define UINT16BEU pt_uint16oeu_t
   #define UINT32BEU pt_uint32oeu_t
   #define UINT64BEU pt_uint64oeu_t
   #define UUIDLE  pt_uuid_t
   #define UUIDLEU pt_uuidu_t
   #define UUIDBE  pt_uuidoe_t
   #define UUIDBEU pt_uuidoeu_t
#else
   typedef     int16_t     pt_int16be_t ;
   typedef     int32_t     pt_int32be_t ;
   typedef     int64_t     pt_int64be_t ;
   typedef    uint16_t    pt_uint16be_t ;
   typedef    uint32_t    pt_uint32be_t ;
   typedef    uint64_t    pt_uint64be_t ;
   typedef  pt_int16u_t    pt_int16beu_t;
   typedef  pt_int32u_t    pt_int32beu_t;
   typedef  pt_int64u_t    pt_int64beu_t;
   typedef pt_uint16u_t   pt_uint16beu_t;
   typedef pt_uint32u_t   pt_uint32beu_t;
   typedef pt_uint64u_t   pt_uint64beu_t;
   typedef  pt_int16oe_t   pt_int16le_t ;
   typedef  pt_int32oe_t   pt_int32le_t ;
   typedef  pt_int64oe_t   pt_int64le_t ;
   typedef pt_uint16oe_t  pt_uint16le_t ;
   typedef pt_uint32oe_t  pt_uint32le_t ;
   typedef pt_uint64oe_t  pt_uint64le_t ;
   typedef  pt_int16oeu_t  pt_int16leu_t;
   typedef  pt_int32oeu_t  pt_int32leu_t;
   typedef  pt_int64oeu_t  pt_int64leu_t;
   typedef pt_uint16oeu_t pt_uint16leu_t;
   typedef pt_uint32oeu_t pt_uint32leu_t;
   typedef pt_uint64oeu_t pt_uint64leu_t;
   typedef pt_uuid_t    pt_uuidbe_t ;
   typedef pt_uuidu_t   pt_uuidbeu_t;
   typedef pt_uuidoe_t  pt_uuidle_t ;
   typedef pt_uuidoeu_t pt_uuidleu_t;
   #define  INT16BE      int16_t
   #define  INT32BE      int32_t
   #define  INT64BE      int64_t
   #define UINT16BE     uint16_t
   #define UINT32BE     uint32_t
   #define UINT64BE     uint64_t
   #define  INT16BEU  pt_int16u_t
   #define  INT32BEU  pt_int32u_t
   #define  INT64BEU  pt_int64u_t
   #define UINT16BEU pt_uint16u_t
   #define UINT32BEU pt_uint32u_t
   #define UINT64BEU pt_uint64u_t
   #define  INT16LE   pt_int16oe_t
   #define  INT32LE   pt_int32oe_t
   #define  INT64LE   pt_int64oe_t
   #define UINT16LE  pt_uint16oe_t
   #define UINT32LE  pt_uint32oe_t
   #define UINT64LE  pt_uint64oe_t
   #define  INT16LEU  pt_int16oeu_t
   #define  INT32LEU  pt_int32oeu_t
   #define  INT64LEU  pt_int64oeu_t
   #define UINT16LEU pt_uint16oeu_t
   #define UINT32LEU pt_uint32oeu_t
   #define UINT64LEU pt_uint64oeu_t
   #define UUIDBE  pt_uuid_t
   #define UUIDBEU pt_uuidu_t
   #define UUIDLE  pt_uuidoe_t
   #define UUIDLEU pt_uuidoeu_t
#endif
#define UINT24U PT_UINT24U
#define UINT24LEU PT_UINT24LEU
#define UINT24BEU PT_UINT24BEU

   // Unions for strict-aliasing safe conversions.
PT_TYPE_DEFINE(pt_uint32_2_t) { uint32_t d[2]; };
PT_TYPEUNION_DEFINE(pt_union_int16_t) { int16_t a; pt_int16u_t u; pt_int16oe_t oe; pt_int16oeu_t oeu; pt_int16le_t le; pt_int16leu_t leu; pt_int16be_t be; pt_int16beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_int32_t) { int32_t a; pt_int32u_t u; pt_int32oe_t oe; pt_int32oeu_t oeu; pt_int32le_t le; pt_int32leu_t leu; pt_int32be_t be; pt_int32beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_int64_t) { int64_t a; pt_int64u_t u; pt_int64oe_t oe; pt_int64oeu_t oeu; pt_int64le_t le; pt_int64leu_t leu; pt_int64be_t be; pt_int64beu_t beu; pt_uint32_2_t h; };
PT_TYPEUNION_DEFINE(pt_union_int16u_t) { pt_int16u_t u; pt_int16oeu_t oeu; pt_int16leu_t leu; pt_int16beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_int32u_t) { pt_int32u_t u; pt_int32oeu_t oeu; pt_int32leu_t leu; pt_int32beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_int64u_t) { pt_int64u_t u; pt_int64oeu_t oeu; pt_int64leu_t leu; pt_int64beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_uint16_t) { uint16_t a; pt_uint16u_t u; pt_uint16oe_t oe; pt_uint16oeu_t oeu; pt_uint16le_t le; pt_uint16leu_t leu; pt_uint16be_t be; pt_uint16beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_uint32_t) { uint32_t a; pt_uint32u_t u; pt_uint32oe_t oe; pt_uint32oeu_t oeu; pt_uint32le_t le; pt_uint32leu_t leu; pt_uint32be_t be; pt_uint32beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_uint64_t) { uint64_t a; pt_uint64u_t u; pt_uint64oe_t oe; pt_uint64oeu_t oeu; pt_uint64le_t le; pt_uint64leu_t leu; pt_uint64be_t be; pt_uint64beu_t beu; pt_uint32_2_t h; };
PT_TYPEUNION_DEFINE(pt_union_uint16u_t) { pt_uint16u_t u; pt_uint16oeu_t oeu; pt_uint16leu_t leu; pt_uint16beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_uint32u_t) { pt_uint32u_t u; pt_uint32oeu_t oeu; pt_uint32leu_t leu; pt_uint32beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_uint64u_t) { pt_uint64u_t u; pt_uint64oeu_t oeu; pt_uint64leu_t leu; pt_uint64beu_t beu; };
PT_TYPEUNION_DEFINE(pt_union_uuid_t) { pt_uuid_t a; pt_uuidoe_t oe; pt_uuidu_t u; pt_uuidoeu_t oeu; PT_UUIDLE le; PT_UUIDLEU leu; PT_UUIDBE be; PT_UUIDBEU beu; uint8_t d[16]; };
PT_TYPEUNION_DEFINE(pt_union_uuidu_t) { pt_uuidu_t u; pt_uuidoeu_t oeu; PT_UUIDLEU leu; PT_UUIDBEU beu; uint8_t d[16]; };

PT_INLINE   int16_t   getint16u( const   pt_int16u_t* s) {   pt_int16oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int32_t   getint32u( const   pt_int32u_t* s) {   pt_int32oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int64_t   getint64u( const   pt_int64u_t* s) {   pt_int64oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint16_t  getuint16u( const  pt_uint16u_t* s) {  pt_uint16oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint32_t  getuint32u( const  pt_uint32u_t* s) {  pt_uint32oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint64_t  getuint64u( const  pt_uint64u_t* s) {  pt_uint64oe_t t; t.u = *s; return t.oe; }
#ifdef PT_LITTLEENDIAN
PT_INLINE   int16_t   getint16le(  const       int16_t*  s) { return *s; }
PT_INLINE   int32_t   getint32le(  const       int32_t*  s) { return *s; }
PT_INLINE   int64_t   getint64le(  const       int64_t*  s) { return *s; }
PT_INLINE  uint16_t  getuint16le(  const      uint16_t*  s) { return *s; }
PT_INLINE  uint32_t  getuint32le(  const      uint32_t*  s) { return *s; }
PT_INLINE  uint64_t  getuint64le(  const      uint64_t*  s) { return *s; }
PT_INLINE   int16_t   getint16leu( const    pt_int16u_t* s) {   pt_int16oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int32_t   getint32leu( const    pt_int32u_t* s) {   pt_int32oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int64_t   getint64leu( const    pt_int64u_t* s) {   pt_int64oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint16_t  getuint16leu( const   pt_uint16u_t* s) {  pt_uint16oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint32_t  getuint32leu( const   pt_uint32u_t* s) {  pt_uint32oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint64_t  getuint64leu( const   pt_uint64u_t* s) {  pt_uint64oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int16_t   getint16be(  const    pt_int16oe_t* s) {   int16_t t = s->oe;  return   pt_swapint16(t); }
PT_INLINE   int32_t   getint32be(  const    pt_int32oe_t* s) {   int32_t t = s->oe;  return   pt_swapint32(t); }
PT_INLINE   int64_t   getint64be(  const    pt_int64oe_t* s) {   int64_t t = s->oe;  return   pt_swapint64(t); }
PT_INLINE  uint16_t  getuint16be(  const   pt_uint16oe_t* s) {  uint16_t t = s->oe;  return  pt_swapuint16(t); }
PT_INLINE  uint32_t  getuint32be(  const   pt_uint32oe_t* s) {  uint32_t t = s->oe;  return  pt_swapuint32(t); }
PT_INLINE  uint64_t  getuint64be(  const   pt_uint64oe_t* s) {  uint64_t t = s->oe;  return  pt_swapuint64(t); }
PT_INLINE   int16_t   getint16beu( const   pt_int16oeu_t* s) {   pt_int16oe_t t; t.u = s->u; return   pt_swapint16(t.oe); }
PT_INLINE   int32_t   getint32beu( const   pt_int32oeu_t* s) {   pt_int32oe_t t; t.u = s->u; return   pt_swapint32(t.oe); }
PT_INLINE   int64_t   getint64beu( const   pt_int64oeu_t* s) {   pt_int64oe_t t; t.u = s->u; return   pt_swapint64(t.oe); }
PT_INLINE  uint16_t  getuint16beu( const  pt_uint16oeu_t* s) {  pt_uint16oe_t t; t.u = s->u; return  pt_swapuint16(t.oe); }
PT_INLINE  uint32_t  getuint32beu( const  pt_uint32oeu_t* s) {  pt_uint32oe_t t; t.u = s->u; return  pt_swapuint32(t.oe); }
PT_INLINE  uint64_t  getuint64beu( const  pt_uint64oeu_t* s) {  pt_uint64oe_t t; t.u = s->u; return  pt_swapuint64(t.oe); }
#else
PT_INLINE   int16_t   getint16le(  const    pt_int16oe_t* s) {   int16_t t = s->oe;  return   pt_swapint16(t); }
PT_INLINE   int32_t   getint32le(  const    pt_int32oe_t* s) {   int32_t t = s->oe;  return   pt_swapint32(t); }
PT_INLINE   int64_t   getint64le(  const    pt_int64oe_t* s) {   int64_t t = s->oe;  return   pt_swapint64(t); }
PT_INLINE  uint16_t  getuint16le(  const   pt_uint16oe_t* s) {  uint16_t t = s->oe;  return  pt_swapuint16(t); }
PT_INLINE  uint32_t  getuint32le(  const   pt_uint32oe_t* s) {  uint32_t t = s->oe;  return  pt_swapuint32(t); }
PT_INLINE  uint64_t  getuint64le(  const   pt_uint64oe_t* s) {  uint64_t t = s->oe;  return  pt_swapuint64(t); }
PT_INLINE   int16_t   getint16leu( const   pt_int16oeu_t* s) {   pt_int16oe_t t; t.u = s->u; return   pt_swapint16(t.oe); }
PT_INLINE   int32_t   getint32leu( const   pt_int32oeu_t* s) {   pt_int32oe_t t; t.u = s->u; return   pt_swapint32(t.oe); }
PT_INLINE   int64_t   getint64leu( const   pt_int64oeu_t* s) {   pt_int64oe_t t; t.u = s->u; return   pt_swapint64(t.oe); }
PT_INLINE  uint16_t  getuint16leu( const  pt_uint16oeu_t* s) {  pt_uint16oe_t t; t.u = s->u; return  pt_swapuint16(t.oe); }
PT_INLINE  uint32_t  getuint32leu( const  pt_uint32oeu_t* s) {  pt_uint32oe_t t; t.u = s->u; return  pt_swapuint32(t.oe); }
PT_INLINE  uint64_t  getuint64leu( const  pt_uint64oeu_t* s) {  pt_uint64oe_t t; t.u = s->u; return  pt_swapuint64(t.oe); }
PT_INLINE   int16_t   getint16be(  const    int16_t*  s) { return *s; }
PT_INLINE   int32_t   getint32be(  const    int32_t*  s) { return *s; }
PT_INLINE   int64_t   getint64be(  const    int64_t*  s) { return *s; }
PT_INLINE  uint16_t  getuint16be(  const   uint16_t*  s) { return *s; }
PT_INLINE  uint32_t  getuint32be(  const   uint32_t*  s) { return *s; }
PT_INLINE  uint64_t  getuint64be(  const   uint64_t*  s) { return *s; }
PT_INLINE   int16_t   getint16beu( const    pt_int16u_t* s) {   pt_int16oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int32_t   getint32beu( const    pt_int32u_t* s) {   pt_int32oe_t t; t.u = *s; return t.oe; }
PT_INLINE   int64_t   getint64beu( const    pt_int64u_t* s) {   pt_int64oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint16_t  getuint16beu( const   pt_uint16u_t* s) {  pt_uint16oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint32_t  getuint32beu( const   pt_uint32u_t* s) {  pt_uint32oe_t t; t.u = *s; return t.oe; }
PT_INLINE  uint64_t  getuint64beu( const   pt_uint64u_t* s) {  pt_uint64oe_t t; t.u = *s; return t.oe; }
#endif
PT_INLINE uint32_t   getuint24u( const     PT_UINT24U* s) { return (PT_SCAST( unsigned, s->hi) << 16) |   getuint16u( &s->low); }
PT_INLINE uint32_t getuint24leu( const pt_uint24leu_t* s) { return (PT_SCAST( unsigned, s->hi) << 16) | getuint16leu( &s->low); }
PT_INLINE uint32_t getuint24beu( const pt_uint24beu_t* s) { return (PT_SCAST( unsigned, s->hi) << 16) | getuint16beu( &s->low); }

PT_INLINE void   setint16u(   pt_int16u_t* d,   int16_t s) { *d = PT_CONTAINEROF(const   pt_union_int16_t,a,&s)->u; }
PT_INLINE void   setint32u(   pt_int32u_t* d,   int32_t s) { *d = PT_CONTAINEROF(const   pt_union_int32_t,a,&s)->u; }
PT_INLINE void   setint64u(   pt_int64u_t* d,   int64_t s) { *d = PT_CONTAINEROF(const   pt_union_int64_t,a,&s)->u; }
PT_INLINE void  setuint16u(  pt_uint16u_t* d,  uint16_t s) { *d = PT_CONTAINEROF(const  pt_union_uint16_t,a,&s)->u; }
PT_INLINE void  setuint32u(  pt_uint32u_t* d,  uint32_t s) { *d = PT_CONTAINEROF(const  pt_union_uint32_t,a,&s)->u; }
PT_INLINE void  setuint64u(  pt_uint64u_t* d,  uint64_t s) { *d = PT_CONTAINEROF(const  pt_union_uint64_t,a,&s)->u; }
#ifdef PT_LITTLEENDIAN
PT_INLINE void   setint16le(   int16_t* d,   int16_t s) { *d = s; }
PT_INLINE void   setint32le(   int32_t* d,   int32_t s) { *d = s; }
PT_INLINE void   setint64le(   int64_t* d,   int64_t s) { *d = s; }
PT_INLINE void  setuint16le(  uint16_t* d,  uint16_t s) { *d = s; }
PT_INLINE void  setuint32le(  uint32_t* d,  uint32_t s) { *d = s; }
PT_INLINE void  setuint64le(  uint64_t* d,  uint64_t s) { *d = s; }
PT_INLINE void   setint16leu(   pt_int16u_t* d,   int16_t s) { *d = PT_CONTAINEROF(const   pt_union_int16_t,a,&s)->u; }
PT_INLINE void   setint32leu(   pt_int32u_t* d,   int32_t s) { *d = PT_CONTAINEROF(const   pt_union_int32_t,a,&s)->u; }
PT_INLINE void   setint64leu(   pt_int64u_t* d,   int64_t s) { *d = PT_CONTAINEROF(const   pt_union_int64_t,a,&s)->u; }
PT_INLINE void  setuint16leu(  pt_uint16u_t* d,  uint16_t s) { *d = PT_CONTAINEROF(const  pt_union_uint16_t,a,&s)->u; }
PT_INLINE void  setuint32leu(  pt_uint32u_t* d,  uint32_t s) { *d = PT_CONTAINEROF(const  pt_union_uint32_t,a,&s)->u; }
PT_INLINE void  setuint64leu(  pt_uint64u_t* d,  uint64_t s) { *d = PT_CONTAINEROF(const  pt_union_uint64_t,a,&s)->u; }
PT_INLINE void   setint16be(   pt_int16oe_t* d,   int16_t s) { d->oe =   pt_swapint16(s); }
PT_INLINE void   setint32be(   pt_int32oe_t* d,   int32_t s) { d->oe =   pt_swapint32(s); }
PT_INLINE void   setint64be(   pt_int64oe_t* d,   int64_t s) { d->oe =   pt_swapint64(s); }
PT_INLINE void  setuint16be(  pt_uint16oe_t* d,  uint16_t s) { d->oe =  pt_swapuint16(s); }
PT_INLINE void  setuint32be(  pt_uint32oe_t* d,  uint32_t s) { d->oe =  pt_swapuint32(s); }
PT_INLINE void  setuint64be(  pt_uint64oe_t* d,  uint64_t s) { d->oe =  pt_swapuint64(s); }
PT_INLINE void   setint16beu(   pt_int16oeu_t* d,   int16_t s) {   pt_int16oe_t t; t.oe =   pt_swapint16(s); d->u = t.u; }
PT_INLINE void   setint32beu(   pt_int32oeu_t* d,   int32_t s) {   pt_int32oe_t t; t.oe =   pt_swapint32(s); d->u = t.u; }
PT_INLINE void   setint64beu(   pt_int64oeu_t* d,   int64_t s) {   pt_int64oe_t t; t.oe =   pt_swapint64(s); d->u = t.u; }
PT_INLINE void  setuint16beu(  pt_uint16oeu_t* d,  uint16_t s) {  pt_uint16oe_t t; t.oe =  pt_swapuint16(s); d->u = t.u; }
PT_INLINE void  setuint32beu(  pt_uint32oeu_t* d,  uint32_t s) {  pt_uint32oe_t t; t.oe =  pt_swapuint32(s); d->u = t.u; }
PT_INLINE void  setuint64beu(  pt_uint64oeu_t* d,  uint64_t s) {  pt_uint64oe_t t; t.oe =  pt_swapuint64(s); d->u = t.u; }
#else
PT_INLINE void   setint16le(   pt_int16oe_t* d,   int16_t s) { d->oe =   pt_swapint16(s); }
PT_INLINE void   setint32le(   pt_int32oe_t* d,   int32_t s) { d->oe =   pt_swapint32(s); }
PT_INLINE void   setint64le(   pt_int64oe_t* d,   int64_t s) { d->oe =   pt_swapint64(s); }
PT_INLINE void  setuint16le(  pt_uint16oe_t* d,  uint16_t s) { d->oe =  pt_swapuint16(s); }
PT_INLINE void  setuint32le(  pt_uint32oe_t* d,  uint32_t s) { d->oe =  pt_swapuint32(s); }
PT_INLINE void  setuint64le(  pt_uint64oe_t* d,  uint64_t s) { d->oe =  pt_swapuint64(s); }
PT_INLINE void   setint16leu(   pt_int16oeu_t* d,   int16_t s) {   pt_int16oe_t t; t.oe =   pt_swapint16(s); d->u = t.u; }
PT_INLINE void   setint32leu(   pt_int32oeu_t* d,   int32_t s) {   pt_int32oe_t t; t.oe =   pt_swapint32(s); d->u = t.u; }
PT_INLINE void   setint64leu(   pt_int64oeu_t* d,   int64_t s) {   pt_int64oe_t t; t.oe =   pt_swapint64(s); d->u = t.u; }
PT_INLINE void  setuint16leu(  pt_uint16oeu_t* d,  uint16_t s) {  pt_uint16oe_t t; t.oe =  pt_swapuint16(s); d->u = t.u; }
PT_INLINE void  setuint32leu(  pt_uint32oeu_t* d,  uint32_t s) {  pt_uint32oe_t t; t.oe =  pt_swapuint32(s); d->u = t.u; }
PT_INLINE void  setuint64leu(  pt_uint64oeu_t* d,  uint64_t s) {  pt_uint64oe_t t; t.oe =  pt_swapuint64(s); d->u = t.u; }
PT_INLINE void   setint16be(   int16_t* d,   int16_t s) { *d = s; }
PT_INLINE void   setint32be(   int32_t* d,   int32_t s) { *d = s; }
PT_INLINE void   setint64be(   int64_t* d,   int64_t s) { *d = s; }
PT_INLINE void  setuint16be(  uint16_t* d,  uint16_t s) { *d = s; }
PT_INLINE void  setuint32be(  uint32_t* d,  uint32_t s) { *d = s; }
PT_INLINE void  setuint64be(  uint64_t* d,  uint64_t s) { *d = s; }
PT_INLINE void   setint16beu(   pt_int16u_t* d,   int16_t s) { *d = PT_CONTAINEROF(const   pt_union_int16_t,a,&s)->u; }
PT_INLINE void   setint32beu(   pt_int32u_t* d,   int32_t s) { *d = PT_CONTAINEROF(const   pt_union_int32_t,a,&s)->u; }
PT_INLINE void   setint64beu(   pt_int64u_t* d,   int64_t s) { *d = PT_CONTAINEROF(const   pt_union_int64_t,a,&s)->u; }
PT_INLINE void  setuint16beu(  pt_uint16u_t* d,  uint16_t s) { *d = PT_CONTAINEROF(const  pt_union_uint16_t,a,&s)->u; }
PT_INLINE void  setuint32beu(  pt_uint32u_t* d,  uint32_t s) { *d = PT_CONTAINEROF(const  pt_union_uint32_t,a,&s)->u; }
PT_INLINE void  setuint64beu(  pt_uint64u_t* d,  uint64_t s) { *d = PT_CONTAINEROF(const  pt_union_uint64_t,a,&s)->u; }
#endif
PT_INLINE void   setuint24u(     PT_UINT24U* d, unsigned s) { d->hi = PT_SCAST( uint8_t, s >> 16);   setuint16u( &d->low, PT_SCAST( uint16_t, s)); }
PT_INLINE void setuint24leu( pt_uint24leu_t* d, unsigned s) { d->hi = PT_SCAST( uint8_t, s >> 16); setuint16leu( &d->low, PT_SCAST( uint16_t, s)); }
PT_INLINE void setuint24beu( pt_uint24beu_t* d, unsigned s) { d->hi = PT_SCAST( uint8_t, s >> 16); setuint16beu( &d->low, PT_SCAST( uint16_t, s)); }

PT_INLINE void   zeroint16u(   pt_int16u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint32u(   pt_int32u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint64u(   pt_int64u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint16u(  pt_uint16u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint32u(  pt_uint32u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint64u(  pt_uint64u_t* d) { memset(d,0,sizeof(*d)); }
#ifdef PT_LITTLEENDIAN
PT_INLINE void   zeroint16le(   int16_t* d) { *d = 0; }
PT_INLINE void   zeroint32le(   int32_t* d) { *d = 0; }
PT_INLINE void   zeroint64le(   int64_t* d) { *d = 0; }
PT_INLINE void  zerouint16le(  uint16_t* d) { *d = 0; }
PT_INLINE void  zerouint32le(  uint32_t* d) { *d = 0; }
PT_INLINE void  zerouint64le(  uint64_t* d) { *d = 0; }
PT_INLINE void   zeroint16leu(   pt_int16u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint32leu(   pt_int32u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint64leu(   pt_int64u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint16leu(  pt_uint16u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint32leu(  pt_uint32u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint64leu(  pt_uint64u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint16be(   pt_int16oe_t*  d) { d->oe = 0; }
PT_INLINE void   zeroint32be(   pt_int32oe_t*  d) { d->oe = 0; }
PT_INLINE void   zeroint64be(   pt_int64oe_t*  d) { d->oe = 0; }
PT_INLINE void  zerouint16be(  pt_uint16oe_t*  d) { d->oe = 0; }
PT_INLINE void  zerouint32be(  pt_uint32oe_t*  d) { d->oe = 0; }
PT_INLINE void  zerouint64be(  pt_uint64oe_t*  d) { d->oe = 0; }
PT_INLINE void   zeroint16beu(   pt_int16oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint32beu(   pt_int32oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint64beu(   pt_int64oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint16beu(  pt_uint16oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint32beu(  pt_uint32oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint64beu(  pt_uint64oeu_t* d) { memset(d,0,sizeof(*d)); }
#else
PT_INLINE void   zeroint16le(   pt_int16oe_t* d) { d->oe = 0; }
PT_INLINE void   zeroint32le(   pt_int32oe_t* d) { d->oe = 0; }
PT_INLINE void   zeroint64le(   pt_int64oe_t* d) { d->oe = 0; }
PT_INLINE void  zerouint16le(  pt_uint16oe_t* d) { d->oe = 0; }
PT_INLINE void  zerouint32le(  pt_uint32oe_t* d) { d->oe = 0; }
PT_INLINE void  zerouint64le(  pt_uint64oe_t* d) { d->oe = 0; }
PT_INLINE void   zeroint16leu(   pt_int16oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint32leu(   pt_int32oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint64leu(   pt_int64oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint16leu(  pt_uint16oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint32leu(  pt_uint32oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint64leu(  pt_uint64oeu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint16be(   int16_t* d) { *d = 0; }
PT_INLINE void   zeroint32be(   int32_t* d) { *d = 0; }
PT_INLINE void   zeroint64be(   int64_t* d) { *d = 0; }
PT_INLINE void  zerouint16be(  uint16_t* d) { *d = 0; }
PT_INLINE void  zerouint32be(  uint32_t* d) { *d = 0; }
PT_INLINE void  zerouint64be(  uint64_t* d) { *d = 0; }
PT_INLINE void   zeroint16beu(   pt_int16u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint32beu(   pt_int32u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void   zeroint64beu(   pt_int64u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint16beu(  pt_uint16u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint32beu(  pt_uint32u_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void  zerouint64beu(  pt_uint64u_t* d) { memset(d,0,sizeof(*d)); }
#endif
PT_INLINE void   zerouint24u(     PT_UINT24U* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void zerouint24leu( pt_uint24leu_t* d) { memset(d,0,sizeof(*d)); }
PT_INLINE void zerouint24beu( pt_uint24beu_t* d) { memset(d,0,sizeof(*d)); }

PT_INLINE void getuuidu(  pt_uuid_t* d,const pt_uuidu_t*   s) { PT_CONTAINEROF(pt_union_uuid_t,a,d)->u = *s; }
#ifdef PT_LITTLEENDIAN
PT_INLINE void getuuidle( pt_uuid_t* d,const pt_uuid_t*   s) { *d = *s; }
PT_INLINE void getuuidleu(pt_uuid_t* d,const pt_uuidu_t*   s) { PT_CONTAINEROF(pt_union_uuid_t,a,d)->u = *s; }
PT_INLINE void getuuidbe( pt_uuid_t* d,const pt_uuidoe_t*  s) { d->Part1 = getuint32be(&s->data1); d->Part2 = getuint16be(&s->data2); d->Part3 = getuint16be(&s->data3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->Part4)->h = PT_CONTAINEROF(const pt_union_uint64_t,u.d[0],s->data4)->h; }
PT_INLINE void getuuidbeu(pt_uuid_t* d,const pt_uuidoeu_t* s) { d->Part1 = getuint32beu(&s->data1); d->Part2 = getuint16beu(&s->data2); d->Part3 = getuint16beu(&s->data3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->Part4)->u = PT_CONTAINEROF(const pt_union_uint64u_t,u.d[0],s->data4)->u; }
#else
PT_INLINE void getuuidle( pt_uuid_t* d,const pt_uuidoe_t*  s) { d->Part1 = getuint32le(&s->data1); d->Part2 = getuint16le(&s->data2); d->Part3 = getuint16le(&s->data3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->Part4)->h = PT_CONTAINEROF(const pt_union_uint64_t,u.d[0],s->data4)->h; }
PT_INLINE void getuuidleu(pt_uuid_t* d,const pt_uuidoeu_t* s) { d->Part1 = getuint32leu(&s->data1); d->Part2 = getuint16leu(&s->data2); d->Part3 = getuint16leu(&s->data3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->Part4)->u = PT_CONTAINEROF(const pt_union_uint64u_t,u.d[0],s->data4)->u; }
PT_INLINE void getuuidbe( pt_uuid_t* d,const pt_uuid_t*   s) { *d = *s; }
PT_INLINE void getuuidbeu(pt_uuid_t* d,const pt_uuidu_t*   s) { PT_CONTAINEROF(pt_union_uuid_t,a,d)->u = *s; }
#endif

PT_INLINE void setuuidu  (pt_uuidu_t*   d,const pt_uuid_t* s) { *d = PT_CONTAINEROF(const pt_union_uuid_t,a,s)->u; }
#ifdef PT_LITTLEENDIAN
PT_INLINE void setuuidle (pt_uuid_t*   d,const pt_uuid_t* s) { *d = *s; }
PT_INLINE void setuuidleu(pt_uuidu_t*   d,const pt_uuid_t* s) { *d = PT_CONTAINEROF(const pt_union_uuid_t,a,s)->u; }
PT_INLINE void setuuidbe (pt_uuidoe_t*  d,const pt_uuid_t* s) { setuint32be(&d->data1,s->Part1); setuint16be(&d->data2,s->Part2); setuint16be(&d->data3,s->Part3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->data4)->h = PT_CONTAINEROF(const pt_union_uint64_t,u.d[0],s->Part4)->h; }
PT_INLINE void setuuidbeu(pt_uuidoeu_t* d,const pt_uuid_t* s) { setuint32beu(&d->data1,s->Part1); setuint16beu(&d->data2,s->Part2); setuint16beu(&d->data3,s->Part3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->data4)->u = PT_CONTAINEROF(const pt_union_uint64u_t,u.d[0],s->Part4)->u; }
#else
PT_INLINE void setuuidle (pt_uuidoe_t*  d,const pt_uuid_t* s) { setuint32le(&d->data1,s->Part1); setuint16le(&d->data2,s->Part2); setuint16le(&d->data3,s->Part3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->data4)->h = PT_CONTAINEROF(const pt_union_uint64_t,u.d[0],s->Part4)->h; }
PT_INLINE void setuuidleu(pt_uuidoeu_t* d,const pt_uuid_t* s) { setuint32leu(&d->data1,s->Part1); setuint16leu(&d->data2,s->Part2); setuint16leu(&d->data3,s->Part3); PT_CONTAINEROF(pt_union_uint64_t,u.d[0],d->data4)->u = PT_CONTAINEROF(const pt_union_uint64u_t,u.d[0],s->Part4)->u; }
PT_INLINE void setuuidbe (pt_uuid_t*   d,const pt_uuid_t* s) { *d = *s; }
PT_INLINE void setuuidbeu(pt_uuidu_t*   d,const pt_uuid_t* s) { *d = PT_CONTAINEROF(const pt_union_uuid_t,a,s)->u; }
#endif

PT_EXTERNC_END
#endif
