//---------------------------------------------------------------------------
// Copyright 1997-2024 Joe Lowe
//
// Permission is granted to any person obtaining a copy of this Software,
// to deal in the Software without restriction, including the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and sell copies of
// the Software.
//
// The above copyright and permission notice must be left intact in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS WITHOUT WARRANTY.
//---------------------------------------------------------------------------
// file name:  ptcompiler.h
// created:    2013.01.30
//    Restore some compiler sanity. This should be included
//    prior to any system or runtime headers.
//---------------------------------------------------------------------------
#ifndef PTCOMPILER_H
#define PTCOMPILER_H
#include "ptpublic.h"
PT_EXTERNC_START

// #define MAKE_FEATURE_KERNEL_TRAPS

#if defined(__KERNEL__) && !defined(KERNEL)
#define KERNEL
#endif

#undef PT_DEBUG
#if defined(DEBUG) || (defined(DBG) && DBG) || defined(_DEBUG)
   #define PT_DEBUG
      // NDEBUG and _DEBUG controls CRT debugging features, which
      // can cause linking and runtime issues if not uniformly
      // applied.
   #ifndef _DEBUG
   #define _DEBUG
   #endif
   #undef NDEBUG
#else
   #undef NDEBUG
   #define NDEBUG
#endif

#undef LITTLEENDIAN
#undef BIGENDIAN
#undef WORDS_LITTLEENDIAN
#undef WORDS_BIGENDIAN
#ifdef PT_LITTLEENDIAN
#ifdef _WIN32
   // Must match conflicting definition used in winsock2.h
#define LITTLEENDIAN 0x0001
#else
#define LITTLEENDIAN
#endif
#define WORDS_LITTLEENDIAN
#else
#define BIGENDIAN
#define WORDS_BIGENDIAN
#endif

   // Whatever it takes to enable portable modern platform and
   // compiler features.
#ifndef _INCLUDE__STDC_A1_SOURCE
#define _INCLUDE__STDC_A1_SOURCE
#endif
#ifndef _XOPEN_SOURCE
#define _XOPEN_SOURCE
#endif
#ifndef _XOPEN_SOURCE_EXTENDED
#define _XOPEN_SOURCE_EXTENDED
#endif
#ifndef _BSD_SOURCE
#define _BSD_SOURCE
#endif
#ifndef _SVID_SOURCE
#define _SVID_SOURCE
#endif
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#ifndef _REENTRANT
#define _REENTRANT
#endif
#ifndef _THREAD_SAFE
#define _THREAD_SAFE
#endif
#ifndef _LARGEFILE_SOURCE
#define _LARGEFILE_SOURCE
#endif
#ifndef _LARGEFILE64_SOURCE
#define _LARGEFILE64_SOURCE
#endif
#ifndef _FILE_OFFSET_BITS
#define _FILE_OFFSET_BITS 64
#endif
#ifndef _DARWIN_C_SOURCE
#define _DARWIN_C_SOURCE
#endif
#ifndef KERNEL
#ifndef __DARWIN_64_BIT_INO_T
#define __DARWIN_64_BIT_INO_T 1
#endif
#ifndef __DARWIN_ONLY_64_BIT_INO_T
#define __DARWIN_ONLY_64_BIT_INO_T 1
#endif
#endif
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS
#endif
#ifndef __STDC_CONSTANT_MACROS
#define __STDC_CONSTANT_MACROS
#endif
#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS
#endif
#ifndef _CRT_NONSTDC_NO_WARNINGS
#define _CRT_NONSTDC_NO_WARNINGS
#endif
#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif
#ifndef _CRT_SECURE_NO_DEPRECATE
#define _CRT_SECURE_NO_DEPRECATE
#endif
#undef _FORTIFY_SOURCE
#ifndef _ALL_SOURCE
#define _ALL_SOURCE 1
#endif
#ifndef _POSIX_PTHREAD_SEMANTICS
#define _POSIX_PTHREAD_SEMANTICS 1
#endif
#ifndef _TANDEM_SOURCE
#define _TANDEM_SOURCE 1
#endif
#ifndef __EXTENSIONS__
#define __EXTENSIONS__ 1
#endif
#ifndef _MT
#define _MT
#endif
#ifndef __BSD_VISIBLE
#define __BSD_VISIBLE 1
#endif
#ifndef _DARWIN_UNLIMITED_STREAMS
#define _DARWIN_UNLIMITED_STREAMS
#endif
   // The following MSVC checks are understandable, but when
   // building public libraries with controlled dependencies,
   // the checks can cause false incompatibities with client
   // code or other libraries.
#ifndef _ALLOW_MSC_VER_MISMATCH
#define _ALLOW_MSC_VER_MISMATCH
#endif
#ifndef _ALLOW_ITERATOR_DEBUG_LEVEL_MISMATCH
#define _ALLOW_ITERATOR_DEBUG_LEVEL_MISMATCH
#endif
#ifndef _ALLOW_RUNTIME_LIBRARY_MISMATCH
#define _ALLOW_RUNTIME_LIBRARY_MISMATCH
#endif

#ifdef _DEBUG
#ifndef _SECURE_SCL
#define _SECURE_SCL 1
#endif
#ifndef _HAS_ITERATOR_DEBUGGING
#define _HAS_ITERATOR_DEBUGGING 1
#endif
#ifndef _ITERATOR_DEBUG_LEVEL
#define _ITERATOR_DEBUG_LEVEL 2
#endif
#else
#ifndef _SECURE_SCL
#define _SECURE_SCL 0
#endif
#ifndef _HAS_ITERATOR_DEBUGGING
#define _HAS_ITERATOR_DEBUGGING 0
#endif
#ifndef _ITERATOR_DEBUG_LEVEL
#define _ITERATOR_DEBUG_LEVEL 0
#endif
#endif
#ifndef _DEFAULT_SOURCE
#define _DEFAULT_SOURCE
#endif
#ifndef NOMINMAX
#define NOMINMAX
#endif

#ifdef DEBUG
   #undef PT_INLINE
   #define PT_INLINE PT_INLINE2
   #undef PT_INLINE_BOOL
   #define PT_INLINE_BOOL PT_INLINE2 PT_BOOL
#endif

#if defined(_MSC_VER)

      // Disabling some less useful warnings makes warning
      // level 4 usable.

      // this reference in base constructor list
   #pragma warning(disable: 4355)
      // inline assembler optimization
   #pragma warning(disable: 4704)
      // nameless structure/union
   #pragma warning(disable: 4201)
      // unreferenced parameter
   #pragma warning(disable: 4100)
      // unused inline functions
   #pragma warning(disable: 4514)
      // unexpanded inline functions
   #pragma warning(disable: 4505)
      // unused local functions
   #pragma warning(disable: 4710)
      // unreachable code
   #pragma warning(disable: 4702)
      // can not generate copy constructor
   #pragma warning(disable: 4511)
      // can not generate assignment operator
   #pragma warning(disable: 4512)
      // conditional expression constant
   #pragma warning(disable: 4127)
      // structure was padded due to __declspec(align())
   #pragma warning(disable: 4324)
      // switch statement contains only default label
   #pragma warning(disable: 4065)
      // function compiled as native
   #pragma warning(disable: 4793)
      // function selected for automatic inline expansion
   #pragma warning(disable: 4711)
      // non-int bit field types
   #pragma warning(disable: 4214)
      // named type definition in parentheses
   #pragma warning(disable: 4115)
      // malloc uses old-style declarator
   #pragma warning(disable: 4131)
      // non-constant aggregate initializer
   #pragma warning(disable: 4204)
      // truncation from double to float
   #pragma warning(disable: 4305)
      // conversion from double to float
   #pragma warning(disable: 4244)
      // symbol was previously declared with declspec...
   #pragma warning(disable: 4565)
      // sizeof(void)
   #pragma warning(disable: 4034)
      // unnamed type definition in parentheses
   #pragma warning(disable: 4116)
      // initialization with address of automatic variable
   #pragma warning(disable: 4221)
      // __forceinline not inlined
   #pragma warning(disable: 4714)
      // function/data ptr conversion
   #pragma warning(disable: 4152)
      // not available as an intrinsic function
   #pragma warning(disable: 4163)
      // flexible array warning
   #pragma warning(disable: 4200)
      // no exception handling
   #pragma warning(disable: 4530)
      // c-linkage returns class
   #pragma warning(disable: 4190)
      // no matching delete operator
   #pragma warning(disable: 4291)
      // const should be initialized
   #pragma warning(disable: 4132)
      // consider arch:AVX
   #pragma warning(disable: 4752)
      // defined operator in output of macro expansion
   #pragma warning(disable: 5105)

      // Disable some analyze warnings that are
      // unhelpful, broken, or misused in sys headers.

      // inconsistent annotation (fascist)
   #pragma warning( disable: 28251 28252 28253 28301 28204)
      // broken annotation (version compat?)
   #pragma warning( disable: 28285)
      // using EXCEPTION_EXECUTE_HANDLER (fascist)
   #pragma warning( disable: 6320)
      // empty _except block (fascist)
   #pragma warning( disable: 6322)
      // non atomic read of atomic written memory (fascist)
   #pragma warning( disable: 28112)
      // use of discouraged function (fascist)
   #pragma warning( disable: 28159 6258)
      // possible dereference of null ptr (broken)
   #pragma warning( disable: 28182 6011 6387)
      // possible memory leak (broken)
   #pragma warning( disable: 6014 28197)
      // possible buffer bounds issue (broken)
   #pragma warning( disable: 6385 6386)
      // return value ignored (fascist, misused)
   #pragma warning( disable: 6031)
      // semantically different integer types (unhelpful)
   #pragma warning( disable: 6217)
      // redundant code (unhelpful)
   #pragma warning( disable: 6287)
      // use of arithmetic operator with bool type (unhelpful)
   #pragma warning( disable: 6323)
      // comparison of two constants (unhelpful)
   #pragma warning( disable: 6326)
      // sizeof * sizeof (broken)
   #pragma warning( disable: 6260)
      // 32 bit shift in 64 bit experession (broken)
   #pragma warning( disable: 6297)
   #if _MSC_VER < 1900
         // wrong parameter type passed to function (broken)
      #pragma warning( disable: 6330)
         // mixed use of sizeof and countof (broken)
      #pragma warning( disable: 6305)
         // possible use of uninitialized memory (broken, misused)
      #pragma warning( disable: 6001)
         // sizeof expression contains operator (fascist)
      #pragma warning( disable: 6334)
   #endif
      // _alloca can raise stack overflow exception (unhelpful)
   #pragma warning( disable: 6255)
      // realloc might return null pointer (fascist, unhelpful)
   #pragma warning( disable: 6308)
      // annotation mismatches code (misused)
   #pragma warning( disable: 28194)
      // must succeed pool allocations are forbidden (broken)
   #pragma warning( disable: 28160)
      // annotation does not apply to value type
   #pragma warning( disable: 6553)
      // deprecated function used
   #pragma warning( disable: 4996)

      // Startup functions, execute before main.
   #ifdef __cplusplus
      #define PT_INITIALIZER(f) void f(); struct f##_t_ { f##_t_(void) { f(); } }; f##_t_ f##_; void f()
   #else
      #pragma section(".CRT$XCU",read)
      #define PT_INITIALIZER_(f,p) void f(void); __declspec(allocate(".CRT$XCU")) void(*f##_)(void) = f; __pragma(comment(linker,"-include:" p #f "_")) void f(void)
      #ifdef _WIN64
         #define PT_INITIALIZER(f) PT_INITIALIZER_(f,"")
      #else
         #define PT_INITIALIZER(f) PT_INITIALIZER_(f,"_")
      #endif
   #endif

   #define PT_UNREACHABLE() __assume(0)
   #if defined(_M_IX86)
      #define pt_debugtrap_() do { __asm int 3 __asm { } } while(0)
   #else
      #define pt_debugtrap_() do { __debugbreak(); } while(0)
   #endif
   #define pt_debugtrap() pt_debugtrap_()

   #if defined __cplusplus && __cplusplus >= 201703
      #define PT_FALLTHROUGH [[fallthrough]]
   #endif

   #define PT_ALIGNAS(n) __declspec(align(n))

#else
   #ifdef __clang__

      #ifndef PT_CLANG_VER
         #ifndef __apple_build_version__
            #define PT_CLANG_VER (__clang_major__*10000+__clang_minor__*100+__clang_patchlevel__)
         #else
            #define PT_APPLE_CLANG_VER (__clang_major__*10000+__clang_minor__*100+__clang_patchlevel__)
            #if   PT_APPLE_CLANG_VER >= 120000
               #define PT_CLANG_VER          100000
            #elif PT_APPLE_CLANG_VER >= 110400
               #define PT_CLANG_VER           90000
            #elif PT_APPLE_CLANG_VER >= 110000
               #define PT_CLANG_VER           80000
            #elif PT_APPLE_CLANG_VER >= 100200
               #define PT_CLANG_VER           70000
            #elif PT_APPLE_CLANG_VER >= 100000
               #define PT_CLANG_VER           60100
            #elif PT_APPLE_CLANG_VER >=  90300
               #define PT_CLANG_VER           50200
            #elif PT_APPLE_CLANG_VER >=  90000
               #define PT_CLANG_VER           40000
            #elif PT_APPLE_CLANG_VER >=  80000
               #define PT_CLANG_VER           30900
            #elif PT_APPLE_CLANG_VER >=  70300
               #define PT_CLANG_VER           30800
            #else
               #define PT_CLANG_VER           30700
            #endif
         #endif
      #endif

         // Disable some less useful or broken warnings.
      #pragma clang diagnostic ignored "-Wdeprecated-declarations"
      #pragma clang diagnostic ignored "-Wdelete-non-virtual-dtor"
      #pragma clang diagnostic ignored "-Wunused-function"
      #pragma clang diagnostic ignored "-Wincompatible-pointer-types-discards-qualifiers"
      #pragma clang diagnostic ignored "-Wconstant-logical-operand"
      #pragma clang diagnostic ignored "-Winvalid-offsetof"
      #pragma clang diagnostic ignored "-Wunused-private-field"
      #pragma clang diagnostic ignored "-Wparentheses-equality"
      #pragma clang diagnostic ignored "-Wunused-const-variable"
      #pragma clang diagnostic ignored "-Wreorder"
      #pragma clang diagnostic ignored "-Wstring-plus-int"
      #pragma clang diagnostic ignored "-Wc++11-extensions"
      #pragma clang diagnostic ignored "-Wc++1y-extensions"
      #pragma clang diagnostic ignored "-Wtautological-compare"
      #pragma clang diagnostic ignored "-Woverloaded-virtual"
      #pragma clang diagnostic ignored "-Wreturn-type-c-linkage"
      #if PT_CLANG_VER >= 80000
         #pragma clang diagnostic ignored "-Wunused-local-typedef"
      #endif
      #if PT_CLANG_VER >= 100000
         #pragma clang diagnostic ignored "-Wsign-conversion"
         #pragma clang diagnostic error "-Wimplicit-int-conversion"
         #pragma clang diagnostic error "-Wshorten-64-to-32"
      #endif

      #if PT_CLANG_VER >= 50000
         #ifdef __cplusplus
            #define PT_FALLTHROUGH [[clang::__fallthrough__]]
         #else
            // #define PT_FALLTHROUGH __attribute__((__fallthrough__))
         #endif
      #endif

   #elif defined(__GNUC__)

      #ifndef PT_GCC_VER
         #define PT_GCC_MAJOR __GNUC__
         #ifdef __GNUC_MINOR__
            #define PT_GCC_MINOR __GNUC_MINOR__
         #else
            #define PT_GCC_MINOR 0
         #endif
         #ifdef __GNUC_PATCHLEVEL__
            #define PT_GCC_SUBMINOR __GNUC_PATCHLEVEL__
         #else
            #define PT_GCC_SUBMINOR 0
         #endif
         #define PT_GCC_VER (PT_GCC_MAJOR*10000+PT_GCC_MINOR*100+PT_GCC_SUBMINOR)
      #endif
      #if (defined(__APPLE__) && PT_GCC_VER < 40200) || PT_GCC_VER < 40100
         #error ERROR: need GCC 4.1+ or Apple GCC 4.2+
      #endif

         // Disable some less useful or broken warnings.
      #pragma GCC diagnostic ignored "-Wnonnull"
      #if PT_GCC_VER >= 40200
         #pragma GCC diagnostic ignored "-Wunused-function"
         #pragma GCC diagnostic ignored "-Wunused-value"
         #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
      #endif
      #if PT_GCC_VER >= 40400
         #pragma GCC diagnostic ignored "-Warray-bounds"
      #endif
      #if PT_GCC_VER >= 40800
         #pragma GCC diagnostic ignored "-Wunused-local-typedefs"
      #endif
      #if PT_GCC_VER >= 60000
         // #pragma GCC diagnostic ignored "-Wmisleading-indentation"
      #endif
      #if PT_GCC_VER >= 80000
         #pragma GCC diagnostic error "-Walloc-size-larger-than=18446744073709551615"
      #endif
      #if PT_GCC_VER >= 100000
         #pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
      #endif
      #ifndef __cplusplus
         #if PT_GCC_VER >= 40700
            #pragma GCC diagnostic ignored "-Wenum-compare"
         #endif
      #else
         #pragma GCC diagnostic ignored "-Winvalid-offsetof"
         #pragma GCC diagnostic ignored "-Wnon-virtual-dtor"
         #pragma GCC diagnostic ignored "-Wctor-dtor-privacy"
         #pragma GCC diagnostic ignored "-Wreorder"
         #if PT_GCC_VER >= 40700
            #pragma GCC diagnostic ignored "-Wdelete-non-virtual-dtor"
         #endif
         #if PT_GCC_VER >= 60000
            #pragma GCC diagnostic ignored "-Wattributes"
         #endif
      #endif

         // Use PT_PCAST to silence type-punned ptr cast warnings.
      PT_INLINE char* ptbase_pcast(void* s) { return PT_SCAST(char*,s); }
      #define PT_PCAST(t,e) PT_RCAST(t,ptbase_pcast(e))

      #define PT_OFFSETOF(s,m) __builtin_offsetof(s,m)

      #if PT_GCC_VER >= 70000
         #ifdef __cplusplus
            #define PT_FALLTHROUGH [[gnu::__fallthrough__]]
         #else
            #define PT_FALLTHROUGH __attribute__((__fallthrough__))
         #endif
      #endif

   #endif

      // Rest for various compilers that implement GCC
      // language extensions (gcc, clang, ICC).
   #if defined(__GNUC__)

      #if defined(KERNEL) && defined(__linux__) && defined(__i386__)
         #undef PT_CCALL
         #define PT_CCALL __attribute__((__regparm__(0)))
      #endif

         // Startup functions, execute before main.
      #define PT_INITIALIZER(f) static void f(void) __attribute__((__constructor__)); static void f(void)

      #if (! defined __APPLE__) &&  (defined __i386__ || defined __x86_64__)
         #define pt_debugtrap_() do { __asm__("int3"); } while(0)
      #elif defined __clang__
         #define pt_debugtrap_() __builtin_debugtrap()
      #else
         #define pt_debugtrap_() __builtin_trap()
      #endif
      #if defined __APPLE__ && defined KERNEL
         void PT_CCALL pt_connect_debugger( void);
         #define pt_debugtrap() do { pt_connect_debugger(); pt_debugtrap_(); } while(0)
      #else
         #define pt_debugtrap() pt_debugtrap_()
      #endif

      #define PT_UNREACHABLE() __builtin_unreachable()
      #ifndef __cplusplus
         #define PT_C_COUNTOF(a) (sizeof(a)/sizeof((a)[0])+(sizeof(struct{int:-!!(__builtin_types_compatible_p(__typeof__(a),__typeof__(&a[0])));})))
      #endif

   #endif

   #define PT_ALIGNAS(n) __attribute__((__aligned__(n)))

#endif

#ifndef PT_FALLTHROUGH
   #define PT_FALLTHROUGH do{}while(0)/*fallthrough*/
#endif

#ifndef PT_PCAST
   #define PT_PCAST(t,e) PT_RCAST(t,e)
#endif
#ifndef __cplusplus
   #undef PT_SCAST
   #undef PT_SCASTP
   #undef PT_CCAST
   #undef PT_CCAST2
   #undef PT_CCAST3
   #undef PT_CCASTV
   #undef PT_CCASTV2
   #undef PT_CCASTV3
   #define PT_SCAST(t,e) PT_CAST(t,(e) +!sizeof((*PT_CAST(PT_INT64*,0)=*PT_CAST(PT_INT64*,0)+(e))) +!sizeof((*PT_CAST(PT_INT64*,0)=*PT_CAST(PT_INT64*,0)+(t)1)) )
   #define PT_SCASTP(t,e) PT_CAST(t,sizeof((PT_CAST(struct{const void*a;}*,0)->a=(e)))?(e):0)
   #define PT_CCAST(t,e) PT_CAST(t,(e)+!sizeof((*(e)))+!sizeof(((e)-PT_CAST(t,0))))
   #define PT_CCAST2(t,e) PT_CAST(t,(e)+!sizeof((**(e)))+!sizeof((*(e)-*PT_CAST(t,0))))
   #define PT_CCAST3(t,e) PT_CAST(t,(e)+!sizeof((***(e)))+!sizeof((**(e)-**PT_CAST(t,0))))
   #define PT_CCASTV(t,e) PT_CAST(t,sizeof((PT_CAST(struct{const struct{int a;}*a;}*,0)->a=(e)))?(e):0)
   #define PT_CCASTV2(t,e) PT_CAST(t,sizeof(*(e))+!sizeof((PT_CAST(struct{const struct{int a;}*a;}*,0)->a=*(e)))?(e):0)
   #define PT_CCASTV3(t,e) PT_CAST(t,sizeof(**(e))+!sizeof((PT_CAST(struct{const struct{int a;}*a;}*,0)->a=**(e)))?(e):0)
#endif

#ifndef pt_debugtrap
void PT_CCALL pt_debugtrap(void);
#endif

#define PT_SCASTASSIGN(t,d,s) do { PT_C_ASSERT(sizeof(d) == sizeof(t)); d = PT_SCAST(t,s); } while(0)
#define PT_RCASTASSIGN(t,d,s) do { PT_C_ASSERT(sizeof(d) == sizeof(t)); d = PT_RCAST(t,s); } while(0)

#ifndef PT_C_COUNTOF
   #ifdef __cplusplus
      }
      template <typename t, PT_SIZE_T n> char(&PT_C_COUNTOF_HELPER(t(&)[n]))[n];
      extern "C" {
      #define PT_C_COUNTOF(x) sizeof(PT_C_COUNTOF_HELPER(x))
   #else
      #define PT_C_COUNTOF(a) (sizeof(a)/sizeof((a)[0]))
   #endif
#endif
#define PT_C_ARRAYLEN(a) PT_C_COUNTOF(a)
#define PT_C_ARRAYSIZE(a) (PT_C_COUNTOF(a)*(sizeof((a)[0])))
#define PT_C_STRLEN(s) (PT_C_COUNTOF(s)-1)

#define PT_SIZEOFMEMBER(t,m) sizeof(PT_SCASTP(t*,0)->m)
#define PT_COUNTOFMEMBER(t,m) PT_C_COUNTOF(PT_SCASTP(t*,0)->m)
#ifndef PT_OFFSETOF
#define PT_OFFSETOF(t,m) PT_RCAST(size_t,&(PT_CAST(t*,0)->m))
#endif
#define PT_OFFSETOFEND(t,m) (PT_OFFSETOF(t,m)+PT_SIZEOFMEMBER(t,m))
#define PT_OFFSETOF2(t,m) PT_CAST(size_t,PT_RCAST(const char*,&((t)->m))-PT_RCAST(const char*,t))
#define PT_OFFSETOFEND2(t,m) (PT_OFFSETOF2(t,m)+sizeof((t)->m))
#define PT_OFFSETOF3(t,m,i) (PT_OFFSETOF(t,m)+sizeof(PT_CAST(t*,0)->m[0])*(i))
#define PT_CONTAINEROF(t,m,pm) (sizeof((&PT_CAST(t*,0)->m-(pm)))?PT_PCAST(t*,PT_CCAST(char*,PT_RCAST(const char*,pm)-PT_OFFSETOF(t,m))):0)
#define PT_CONTAINEROF2(t,m,pm) PT_PCAST(t*,PT_CCAST(char*,PT_RCAST(const char*,pm)-PT_OFFSETOF(t,m)))

#define PT_MT_8U32_(c) PT_SCAST(uint32_t,PT_SCAST(uint8_t,c))
#define PT_MT_8U64_(c) PT_SCAST(uint64_t,PT_SCAST(uint8_t,c))
#ifdef PT_LITTLEENDIAN
   #define PT_MAKETAGUINT16(c0,c1)                   PT_SCAST(uint16_t,PT_MT_8U32_(c0)|(PT_MT_8U32_(c1)<<8))
   #define PT_MAKETAGUINT32(c0,c1,c2,c3)             PT_SCAST(uint32_t,PT_MT_8U32_(c0)|(PT_MT_8U32_(c1)<<8)|(PT_MT_8U32_(c2)<<16)|(PT_MT_8U32_(c3)<<24))
   #define PT_MAKETAGUINT64(c0,c1,c2,c3,c4,c5,c6,c7) PT_SCAST(uint64_t,PT_MT_8U32_(c0)|(PT_MT_8U32_(c1)<<8)|(PT_MT_8U32_(c2)<<16)|(PT_MT_8U32_(c3)<<24)|(PT_MT_8U64_(c4)<<32)|(PT_MT_8U64_(c5)<<40)|(PT_MT_8U64_(c6)<<48)|(PT_MT_8U64_(c7)<<56))
#endif
#ifdef PT_BIGENDIAN
   #define PT_MAKETAGUINT16(c0,c1)                   PT_SCAST(uint16_t,PT_MT_8U32_(c1)|(PT_MT_8U32_(c0)<<8))
   #define PT_MAKETAGUINT32(c0,c1,c2,c3)             PT_SCAST(uint32_t,PT_MT_8U32_(c3)|(PT_MT_8U32_(c2)<<8)|(PT_MT_8U32_(c1)<<16)|(PT_MT_8U32_(c0)<<24))
   #define PT_MAKETAGUINT64(c0,c1,c2,c3,c4,c5,c6,c7) PT_SCAST(uint64_t,PT_MT_8U32_(c7)|(PT_MT_8U32_(c6)<<8)|(PT_MT_8U32_(c5)<<16)|(PT_MT_8U32_(c4)<<24)|(PT_MT_8U64_(c3)<<32)|(PT_MT_8U64_(c2)<<40)|(PT_MT_8U64_(c1)<<48)|(PT_MT_8U64_(c0)<<56))
#endif
#define PT_MAKETAGINT16(c0,c1)                   PT_SCAST(int16_t,PT_MAKETAGUINT16(c0,c1))
#define PT_MAKETAGINT32(c0,c1,c2,c3)             PT_SCAST(int32_t,PT_MAKETAGUINT32(c0,c1,c2,c3))
#define PT_MAKETAGINT64(c0,c1,c2,c3,c4,c5,c6,c7) PT_SCAST(int64_t,PT_MAKETAGUINT64(c0,c1,c2,c3,c4,c5,c6,c7))
#define PT_MAKETAGINT(c0,c1,c2,c3) PT_MAKETAGINT32(c0,c1,c2,c3)
#define PT_MAKETAGUINT(c0,c1,c2,c3) PT_MAKETAGUINT32(c0,c1,c2,c3)

PT_PRIVATE void PT_CCALL pt_assert_failed( const char* file, int line, const char* comment);
PT_PRIVATE void PT_CCALL pt_fatal_exit( const char* file, int line, const char* comment);
typedef void (PT_CCALL*pt_assert_handler_t)( const char* comment);
PT_PRIVATE void PT_CCALL pt_set_assert_handler( pt_assert_handler_t);
#define PT_FATAL( comment) pt_fatal_exit( __FILE__, __LINE__, comment)
#undef PT_ASSERT
#ifdef PT_DEBUG
#if defined MAKE_FEATURE_KERNEL_TRAPS || !defined KERNEL
   #define PISMO_TRAP pt_debugtrap()
   #define PT_TRAPIF(expr) do { if(expr) pt_debugtrap(); } while(0)
#else
   #define PISMO_TRAP pt_assert_failed( __FILE__, __LINE__, "TRAP")
   #define PT_TRAPIF(expr) do { if (!(expr)) { pt_assert_failed( __FILE__, __LINE__, "TRAP"); } } while(0)
#endif
   #define PT_ASSERT(expr) do { if (!(expr)) { pt_assert_failed( __FILE__, __LINE__, #expr); } } while(0)
   #define PT_VERIFY(expr) PT_ASSERT(expr)
   #define PT_VERIFYNOT(expr) PT_ASSERT(!(expr))
   #define PT_VERIFYIS(expr,val) PT_ASSERT((expr) == (val))
   #define PT_VERIFYISNOT(expr,val) PT_ASSERT((expr) != (val))
#else
   #define PISMO_TRAP do { } while(0)
   #define PT_TRAPIF(expr) PISMO_TRAP
   #define PT_ASSERT(expr) PISMO_TRAP
   #define PT_VERIFY(expr) expr
   #define PT_VERIFYNOT(expr) expr
   #define PT_VERIFYIS(expr,val) expr
   #define PT_VERIFYISNOT(expr,val) expr
#endif
   // Linux sys headers define PT_TRAP.
#undef PT_TRAP
#define PT_TRAP PISMO_TRAP

static void (*g_pt_need_in_dump_)( const void*, ...);
PT_INLINE void pt_need_in_dump_unused( void) { g_pt_need_in_dump_(0); }
#define PT_NEED_IN_DUMP(...) do { if (g_pt_need_in_dump_) g_pt_need_in_dump_( __VA_ARGS__); } while (0)

PT_EXTERNC_END
#endif
