//----------------------------------------------------------------------------
// Copyright 2001-2023 Joe Lowe
//
// Permission is granted to any person obtaining a copy of this Software,
// to deal in the Software without restriction, including the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and sell copies of
// the Software.
//
// The above copyright and permission notice must be left intact in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS WITHOUT WARRANTY.
//----------------------------------------------------------------------------
// file name:  traceio.h
// created:    2001.06.05
//----------------------------------------------------------------------------
#ifndef TRACEIO_H
#define TRACEIO_H
#include "portableint.h"
#include "traces.h"
PT_EXTERNC_START

#ifdef _WIN32
#define TRACESERVICENAME L"traces"
#define TRACESYSNAME L"traces.sys"
#define TRACEDEVICENAME L"traceio1"
#define TRACEDLLNAME L"traceapp.dll"
#else
#define TRACEDEVICENAME "pttrace"
#ifdef __APPLE__
#define TRACEDLLNAME "libpttrace1.dylib"
#else
#define TRACEDLLNAME "libpttrace1.so"
#endif
#endif
#define TRACEEXPORTSNAME "traceexports1"

void _TRCDECL trautoattach(void);

enum {
   TRACEVERSION                 = 5,
   TRACECLIENTMINCOMPATVERSION  = 5,
   TRACESERVERMINCOMPATVERSION  = 3, };

typedef struct _TRCLIENT *TRCLIENT;

enum { TRACEEXPORTSTAG = PT_MAKETAGINT('T','R','X','T') };

PT_TYPE_DEFINE(TRACEEXPORTS)
{
   int tag;
   int version;
   int minCompatVersion;
   unsigned timeStampLen;
   void* reserved1;
   void* reserved2;
   TRCLIENT (_TRCDECL*trattach)(void);
   void (_TRCDECL*trdetach)(TRCLIENT);
   TRCHANNEL (_TRCDECL*tropen)(TRCLIENT,const char* channelName);
   TRCHANNEL (_TRCDECL*trwopen)(TRCLIENT,const wchar_t* channelName);
   TRCHANNEL (_TRCDECL*trdup)(TRCLIENT,TRCHANNEL);
   void (_TRCDECL*trclose)(TRCLIENT,TRCHANNEL);
   void (_TRCDECL*trdefault)(TRCLIENT,TRCHANNEL);
   size_t (_TRCDECL*trwlist)(TRCLIENT,wchar_t* buffer,size_t maxChars);
   void (_TRCDECL*trinfo)(TRCLIENT,TRCHANNEL,size_t* startLine,
      size_t* endLine,size_t* endLineLength);
   size_t (_TRCDECL*trwread)(TRCLIENT,TRCHANNEL,size_t line,
      wchar_t* buffer,size_t maxChars);
   void (_TRCDECL*trprint)(TRCLIENT,TRCHANNEL,const char* value);
   void (_TRCDECL*trwprint)(TRCLIENT,TRCHANNEL,const wchar_t* value);
   void (_TRCDECL*trvprintf)(TRCLIENT,TRCHANNEL,const char* format,
      va_list args);
   void (_TRCDECL*trvwprintf)(TRCLIENT,TRCHANNEL,const wchar_t* format,
      va_list args);
   void (_TRCDECL*trcrash)(TRCLIENT);
   void (_TRCDECL*trreset)(TRCLIENT,TRCHANNEL);
   size_t (_TRCDECL*trlist)(TRCLIENT,char* buffer,size_t maxChars);
   size_t (_TRCDECL*trread)(TRCLIENT,TRCHANNEL,size_t line,char* buffer,
      size_t maxChars);
   void* reserved3[2];
   void* reserved4[8];
};

extern TRCLIENT traceClient;
extern TRACEEXPORTS* traceExports;
extern int/*bool*/ traceNoDetach;

#ifdef __APPLE__
#define GENIOCTL_INT32 INT32LE
#define GENIOCTL_INT64 INT64LE
#define GENIOCTL_UINT32 UINT32LE
#define GENIOCTL_UINT64 UINT64LE
#define GENIOCTL_SETINT32(d,v) setint32le(d,v)
#define GENIOCTL_SETINT64(d,v) setint64le(d,v)
#define GENIOCTL_SETUINT32(d,v) setuint32le(d,v)
#define GENIOCTL_SETUINT64(d,v) setuint64le(d,v)
#define GENIOCTL_GETINT32(s) getint32le(s)
#define GENIOCTL_GETINT64(s) getint64le(s)
#define GENIOCTL_GETUINT32(s) getuint32le(s)
#define GENIOCTL_GETUINT64(s) getuint64le(s)
#else
#define GENIOCTL_INT32 int32_t
#define GENIOCTL_INT64 int64_t
#define GENIOCTL_UINT32 uint32_t
#define GENIOCTL_UINT64 uint64_t
#define GENIOCTL_SETINT32(d,v) do { *(d) = (v); } while(0)
#define GENIOCTL_SETINT64(d,v) do { *(d) = (v); } while(0)
#define GENIOCTL_SETUINT32(d,v) do { *(d) = (v); } while(0)
#define GENIOCTL_SETUINT64(d,v) do { *(d) = (v); } while(0)
#define GENIOCTL_GETINT32(s) (*(s))
#define GENIOCTL_GETINT64(s) (*(s))
#define GENIOCTL_GETUINT32(s) (*(s))
#define GENIOCTL_GETUINT64(s) (*(s))
#endif

#ifdef _WIN32

// NT IOCTL code bitmap definition with values for vendor specific generic
//  codes
// 0000000001000010 001???????????00
// |              / |/||         /|---- bits 0-1   = METHOD_BUFFERED
// |             /  | ||--------------- bits 2-12  = function
// |            /   | |---------------- bits 13    = vendor specific
// |           /    |------------------ bits 14-15 = FILE_ANY_ACCESS
// |----------------------------------- bits 16-31 = FILE_DEVICE_UNKNOWN
#define GENIOCTLBASE      0x00222000
#define GENIOCTLFUNCMASK  0x00001FFC
#define GENIOCTLFUNCSHIFT 2

#else

typedef struct
{
   GENIOCTL_INT32 code;
   GENIOCTL_INT32 error;
   GENIOCTL_UINT64 input;
   GENIOCTL_UINT64 inputSize;
   GENIOCTL_UINT64 output;
   GENIOCTL_UINT64 maxOutputSize;
   GENIOCTL_UINT64 outputSize;
} GenIoctlRequest;
#ifdef _IOR
#ifdef __APPLE__
#define GENIOCTLVER(t) _IOR('g',0,t)
#define GENIOCTLREQ _IOWR('g',1,GenIoctlRequest)
#define GENIOCTLSIZE(c) IOCPARM_LEN(c)
#define GENIOCTLCHECK(a,b) (IOCBASECMD(a) == IOCBASECMD(b))
#else
#define GENIOCTLVER(t) _IOR('g',0,t)
#define GENIOCTLREQ _IOWR('g',1,GenIoctlRequest)
#define GENIOCTLSIZE(c) _IOC_SIZE(c)
#define GENIOCTLCHECK(a,b) (((a)&~(((1<<_IOC_SIZEBITS)-1)<<_IOC_SIZESHIFT)) == ((b)&~(((1<<_IOC_SIZEBITS)-1)<<_IOC_SIZESHIFT)))
#endif
#endif

#endif

enum {
   TRACEIOGETVERSION = 0,
   TRACEIOOPEN       = 1,
   TRACEIOOPENW      = 2,
   TRACEIODUP        = 3,
   TRACEIOCLOSE      = 4,
   TRACEIOLISTW      = 5,
   TRACEIOINFO       = 6,
   TRACEIOREADW      = 7,
   TRACEIOPRINT      = 8,
   TRACEIOPRINTW     = 9,
   TRACEIODEFAULT    = 10,
   TRACEIORESET      = 11,
   TRACEIOLIST       = 12,
   TRACEIOREAD       = 13,
   TRACEIOCRASH      = 76, };

PT_TYPE_DEFINE(TRACEIOGETVERSIONRESULTS)
{
   GENIOCTL_INT32 version;
   GENIOCTL_INT32 minCompatVersion;
   GENIOCTL_UINT32 timeStampLen;
   uint8_t reserved1[4];
   GENIOCTL_UINT64 exports;
   uint8_t reserved2[8];
};

PT_TYPE_DEFINE(TRACEIOOPENRESULTS)
{
   GENIOCTL_INT64 channel;
};

typedef TRACEIOOPENRESULTS TRACEIOOPENWRESULTS;

PT_TYPE_DEFINE(TRACEIODUPPARAMS)
{
   GENIOCTL_INT64 channel;
};

typedef TRACEIOOPENRESULTS TRACEIODUPRESULTS;

PT_TYPE_DEFINE(TRACEIOCLOSEPARAMS)
{
   GENIOCTL_INT64 channel;
};

typedef TRACEIOCLOSEPARAMS TRACEIODEFAULTPARAMS;

typedef TRACEIOCLOSEPARAMS TRACEIORESETPARAMS;

PT_TYPE_DEFINE(TRACEIOINFOPARAMS)
{
   GENIOCTL_INT64 channel;
};

PT_TYPE_DEFINE(TRACEIOINFORESULTS)
{
   GENIOCTL_UINT64 startLine;
   GENIOCTL_UINT64 endLine;
   GENIOCTL_UINT64 endLineLen;
};

PT_TYPE_DEFINE(TRACEIOREADPARAMS)
{
   GENIOCTL_INT64 channel;
   GENIOCTL_UINT64 line;
};

typedef TRACEIOREADPARAMS TRACEIOREADWPARAMS;

PT_TYPE_DEFINE(TRACEIOPRINTPARAMS)
{
   GENIOCTL_INT64 channel;
   char value[1];
};

PT_TYPE_DEFINE(TRACEIOPRINTWPARAMS)
{
   GENIOCTL_INT64 channel;
   wchar_t value[1];
};

PT_EXTERNC_END
#endif
